#import "devP2PWrapper.h"
#import "P2PConnectedVC.h"
#import "P2PStartedVC.h"
#include "../../../inc/p2p.h"


void p2p_StateChange(devP2Plib::CP2P *p2p, devP2Plib::States state)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];
    
	sprintf(buff, "%s] State changed to %s\r\n", p2p->MyName, p2p->StateText(state));
    [p addLogText:buff];
}
void p2p_NewUPNPMapping(devP2Plib::CP2P *p2p, char *ExtAddress, char *IntAddress, int TCPPort, int UDPPort)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024] = {0};
    
	if (TCPPort) sprintf(buff, "%s] New TCP mapping: %s->%s (%d)\r\n", p2p->MyName, ExtAddress, IntAddress, TCPPort);
	if (UDPPort) sprintf(buff, "%s] New UDP mapping: %s->%s (%d)\r\n", p2p->MyName, ExtAddress, IntAddress, UDPPort);
    
    [p addLogText:buff];
}
void p2p_SearchStart(devP2Plib::CP2P *p2p, char *binds)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];

	sprintf(buff, "%s] started searching for %s at %s (%d)\r\n", p2p->MyName, p2p->PeerName, p2p->MediatorAddress, p2p->MediatorPort);

    [p addLogText:buff];
}
void p2p_SearchDone(devP2Plib::CP2P *p2p, char *peerName, char *peerAddress, char *customData, devP2Plib::Errors error)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];

	sprintf(buff, "%s] Search finished with error %d %s\r\n", peerName, error, p2p->ErrorText(error));
    [p addLogText: buff];
	if (!error)
    {
        if ([p->m_controllerView.navigationController.visibleViewController isKindOfClass:[P2PStartedVC class]])
        {
            P2PStartedVC *ps = (P2PStartedVC *)p->m_controllerView;
            NSString *str = [NSString stringWithUTF8String:peerAddress];
            dispatch_async(dispatch_get_main_queue(), ^(void)
            {
                [ps onSearchDone:str];
            });
        }
		// p2p->Link(peerAddress);
    }
 
}
void p2p_LinkDone(devP2Plib::CP2P *p2p, char *address, int port, devP2Plib::Errors error)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];
    
    if (![p->m_controllerView.navigationController.visibleViewController isKindOfClass:[P2PConnectedVC class]])
    {
        dispatch_async(dispatch_get_main_queue(), ^(void)
        {
            [p->m_controllerView performSegueWithIdentifier:@"P2PConnectedVC" sender:p->m_controllerView];
        });
    }
    
	if (!error)
		sprintf(buff, "%s] Connected to %s (%d)\r\n", p2p->MyName, address, port);
	else
		sprintf(buff, "%s] Connection failed with error %s (%d)\r\n", p2p->MyName, p2p->ErrorText(error), error);
    
    [p addLogText:buff];
}

void p2p_Stopped(devP2Plib::CP2P *p2p, devP2Plib::Errors error)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];
    
	sprintf(buff, "%s] Stopped, error %s\r\n", p2p->MyName, p2p->ErrorText(error));
    [p addLogText:buff];
}

void p2p_TextReceived(devP2Plib::CP2P *p2p, int chanid, char *text)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];
    
	sprintf(buff, "%s] Received text %s\r\n", p2p->MyName, text);
    [p addLogText:buff];
    
    UIAlertView *alert = [[UIAlertView alloc]
                          initWithTitle:[NSString stringWithUTF8String:p2p->PeerName]
                          message:[NSString stringWithUTF8String:text]
                          delegate:nil
                          cancelButtonTitle:@"OK"
                          otherButtonTitles:nil];
    
    [alert performSelectorOnMainThread:@selector(show) withObject:nil waitUntilDone:FALSE];
}

void p2p_DataReceived(devP2Plib::CP2P *p2p, int chanid, char *data, int len)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];
    
	sprintf(buff, "%s] Received %d bytes of data\r\n", p2p->MyName, len);
    [p addLogText:buff];
}
bool p2p_FileSend(devP2Plib::CP2P *p2p, int chanid, char *filename, int64 size)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];
    
	sprintf(buff, "%s] Sending file %s (%ld)\r\n", p2p->MyName, filename, (long)size);
    [p addLogText:buff];
	return true;
}
bool p2p_FileReceive(devP2Plib::CP2P *p2p, int chanid, char *filename, int64 size)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];
    
	sprintf(buff, "%s] Receiving file %s (%ld)\r\n", p2p->MyName, filename, (long)size);
    [p addLogText:buff];
	return true;
}
void p2p_FileProgress(devP2Plib::CP2P *p2p, int chanid, int64 position, int64 size)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];
    
	sprintf(buff, "%s] Progress %ld/%ld (%d%%) bandwidth=%5.2f MB/sec\r\n", p2p->MyName, (long)position, (long)size, (int)((position*100)/size), p2p->Bandwidth(chanid));
    [p addLogText:buff];
}
void p2p_FileDone(devP2Plib::CP2P *p2p, int chanid, devP2Plib::Errors error)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];
    
	sprintf(buff, "%s] Transfer completed with error %d %s\r\n", p2p->MyName, error, p2p->ErrorText(error));
    [p addLogText:buff];
}
bool p2p_ForwardOpen(devP2Plib::CP2P *p2p, int forwid, devP2Plib::ForwardTypes forwardtype, char *localaddress, int localport, char *remoteaddress, int remoteport)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];
    
	const char *t = "Unknown";
	switch (forwardtype)
	{
        case devP2Plib::TCPLocalListen:		t="TCPLocalListen";break;
        case devP2Plib::TCPRemoteListen:	t="TCPRemoteListen";break;
        case devP2Plib::UDPLocalListen:		t="UDPLocalListen";break;
        case devP2Plib::UDPRemoteListen:	t="UDPRemoteListen";break;
        case devP2Plib::TCPSocksProxy:		t="TCPSocksProxy";break;
        case devP2Plib::TCPHttpProxy:		t="TCPHttpProxy";break;
	}
	sprintf(buff, "%s] Forward %s open from %s (%d) ==> %s (%d)\r\n", p2p->MyName, t, localaddress, localport, remoteaddress, remoteport);
    [p addLogText:buff];
	return true;
}
void p2p_ForwardClose(devP2Plib::CP2P *p2p, int forwid, devP2Plib::Errors error)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];
    
	sprintf(buff, "%s] Forward close with error %d %s\r\n", p2p->MyName, error, p2p->ErrorText(error));
    [p addLogText:buff];
}
bool p2p_UserConnected(devP2Plib::CP2P *p2p, int forwid, int chanid, devP2Plib::CP2PForwardUser *user)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];
    
	sprintf(buff, "%s] User connected\r\n", p2p->MyName);
    [p addLogText:buff];
	return true;
}
void p2p_UserDisconnected(devP2Plib::CP2P *p2p, int forwid, int chanid, devP2Plib::CP2PForwardUser *user, devP2Plib::Errors error)
{
    CdevP2PWrapper *p = (__bridge CdevP2PWrapper *)p2p->Tag;
    char buff[1024];
    
	sprintf(buff, "%s] User disconnected with error %d %s\r\n", p2p->MyName, error, p2p->ErrorText(error));
    [p addLogText:buff];
}

@implementation CdevP2PWrapper

-(id)init
{
    self = [super init];
    if (self)
    {
        self->p2p = devP2Plib::CP2P::Create();
        
        devP2Plib::CP2P *p = (devP2Plib::CP2P *)self->p2p;
        p->Tag = (__bridge void *)self;
        
        p->Events.DataReceived = p2p_DataReceived;
        p->Events.FileDone = p2p_FileDone;
        p->Events.FileProgress = p2p_FileProgress;
        p->Events.FileReceive = p2p_FileReceive;
        p->Events.FileSend = p2p_FileSend;
        p->Events.LinkDone = p2p_LinkDone;
        p->Events.NewUPNPMapping = p2p_NewUPNPMapping;
        p->Events.SearchDone = p2p_SearchDone;
        p->Events.SearchStart = p2p_SearchStart;
        p->Events.StateChange = p2p_StateChange;
        p->Events.TextReceived = p2p_TextReceived;
        p->Events.Stopped = p2p_Stopped;
        p->Events.ForwardOpen = p2p_ForwardOpen;
        p->Events.ForwardClose = p2p_ForwardClose;
        p->Events.UserConnected = p2p_UserConnected;
        p->Events.UserDisconnected = p2p_UserDisconnected;
        
        self->p2p_MyName[0] = 0;
        self->p2p_PeerName[0] = 0;
        self->m_logView = NULL;
        self->m_controllerView = NULL;
        self->m_logText = @"";
    }
    return self;
}

-(void)setName: (NSString *) name
{
    devP2Plib::CP2P *p = (devP2Plib::CP2P *)self->p2p;

    strcpy(self->p2p_MyName, [name UTF8String]);
    p->MyName = self->p2p_MyName; // need static buffer
}

-(void)setPeerName: (NSString *)peerName
{
    devP2Plib::CP2P *p = (devP2Plib::CP2P *)self->p2p;

    strcpy(self->p2p_PeerName, [peerName UTF8String]);
    p->PeerName = self->p2p_PeerName;
}

-(void)setPassword: (NSString *)password
{
    devP2Plib::CP2P *p = (devP2Plib::CP2P *)self->p2p;
    p->SetPassword(devP2Plib::EncAES128, (char *)[password UTF8String]);
}

-(void) setOwner:(UIViewController *)controller log:(UITextView *)logView
{
    m_controllerView = controller;
    m_logView = logView;
    [self addLog:@""];
}

-(void) addLog: (NSString *) text
{
    dispatch_async(dispatch_get_main_queue(), ^(void)
    {
        [self addLogSynced:text];
    });
}

-(void) addLogText: (char *) text
{
    [self addLog: [NSString stringWithUTF8String:text]];
}

-(void) addLogSynced:(NSString *)text
{
    if (m_logView)
    {
        if ([text length])
            m_logText = [m_logText stringByAppendingString:text];
        [m_logView setText:m_logText];
        
        NSRange range = NSMakeRange([m_logText length], 0);
        m_logView.scrollEnabled = NO;
        m_logView.scrollEnabled = YES;
        [m_logView scrollRangeToVisible:range];
    }
}

-(BOOL) start
{
    devP2Plib::CP2P *p = (devP2Plib::CP2P *)self->p2p;

    p->TCPPort = 0;
    p->UDPPort = 0;
    int success = p->Start();
    if (success < 0)
    {
        [self addLog:[NSString stringWithFormat:@"%s", p->ErrorText((devP2Plib::Errors)success)]];
        return false;
    }
    
    return true;
}

-(BOOL) search
{
    devP2Plib::CP2P *p = (devP2Plib::CP2P *)self->p2p;
    
    int success = p->Search(NULL);
    if (success < 0)
    {
        [self addLog:[NSString stringWithFormat:@"%s", p->ErrorText((devP2Plib::Errors)success)]];
        return false;
    }
    
    return true;
}
-(BOOL) connect:(NSString *)xml
{
    devP2Plib::CP2P *p = (devP2Plib::CP2P *)self->p2p;
    
    int success = p->Link((char *)[xml UTF8String]);
    if (success < 0)
    {
        [self addLog:[NSString stringWithFormat:@"%s", p->ErrorText((devP2Plib::Errors)success)]];
        return false;
    }
    
    return true;
}

-(void) stop
{
    devP2Plib::CP2P *p = (devP2Plib::CP2P *)self->p2p;
    p->Stop();
}

-(void) sendMessage: (NSString *) text
{
    devP2Plib::CP2P *p = (devP2Plib::CP2P *)self->p2p;
    p->SendText(0, (char *)[text UTF8String]);
}

-(BOOL) sendFile: (NSString *) fileName
{
    devP2Plib::CP2P *p = (devP2Plib::CP2P *)self->p2p;
    int success = p->SendFile(1, (char *)[fileName UTF8String]);
    if (success < 0)
    {
        [self addLog:[NSString stringWithFormat:@"%s", p->ErrorText((devP2Plib::Errors)success)]];
        return false;
    }
    return true;
}


@end
